local addon, private = ...
-- Helper Functions

local function resizeCurrentForRange(current, max)
  local oldCurrent = current:GetText()
  current:SetText(tostring(max)..".000")
  current:ClearWidth()
  current:SetWidth(current:GetWidth())
  current:SetText(oldCurrent)
end

local function RoundValue(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

-- Public Functions

local function SetBorder(self, width, r, g, b, a)
  Library.LibSimpleWidgets.SetBorder(self.current, width, r, g, b, a)
  Library.LibSimpleWidgets.SetBorder(self.editor, width, r, g, b, a)
  Library.LibSimpleWidgets.SetBorder(self.dropdown, width, r, g, b, a)
end

local function SetBackgroundColor(self, r, g, b, a)
  self.current:SetBackgroundColor(r, g, b, a)
  self.slider:SetBackgroundColor(r, g, b, a)
end

local function GetEnabled(self)
  return self.enabled
end

local function SetEnabled(self, enabled)
  assert(type(enabled) == "boolean", "param 1 must be a boolean!")

  self.enabled = enabled
  if enabled then
    self.current:SetFontColor(1, 1, 1, 1)
  else
    self.current:SetFontColor(0.5, 0.5, 0.5, 1)
  end
  if self.editor then
    self.editor:SetVisible(enabled)
    self.current:SetVisible(not enabled)
  end
  self.slider:SetEnabled(enabled)
end

local function GetRange(self)
  return self.slider:GetRange()
end

local function SetRange(self, min, max, step, silent)
  assert(type(min) == "number", "param 1 must be a number!")
  assert(type(max) == "number", "param 2 must be a number!")
  assert(min <= max, "min must be less than or equal to max!")
  assert(silent == nil or type(silent) == "boolean", "param 3 must be a boolean!")

  self.silent = silent ~= nil and silent
  self.slider:SetRange(min, RoundValue(max/step, 0))
  self.step = step
  self.silent = false
  resizeCurrentForRange(self.current, max)
end

local function GetPosition(self)
  return (self.slider:GetPosition()*self.step)
end

local function SetPosition(self, position, silent)
  assert(type(position) == "number", "param 1 must be a number!")
  local min, max = self.slider:GetRange()
  max = RoundValue(max*self.step,0)
  assert(position >= min, "position must be greater than or equal to range minimum")
  assert(position <= max, "position must be less than or equal to range minimum")
  assert(silent == nil or type(silent) == "boolean", "param 2 must be a boolean!")

  local newposition = RoundValue(position/self.step, 0)
  self.silent = silent ~= nil and silent
  self.slider:SetPosition(newposition)
  self.silent = false
  self.current:SetText(tostring(position))
  if self.editor then
    self.editor:SetText(tostring(position))
  end
end

local function ResizeToFit(self)
  self.slider:ClearAll()
  self.current:ClearAll()
  local min, max = self.slider:GetRange()
  max = RoundValue(max*self.step,0)
  resizeCurrentForRange(self.current, max)
  self:SetWidth(self.slider:GetWidth() + self.current:GetWidth())
  self:SetHeight(self.slider:GetHeight() - 2) -- magic numbers!
  self.current:SetPoint("TOPRIGHT", self, "TOPRIGHT")
  self.current:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT")
  self.slider:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 6)
  self.slider:SetPoint("BOTTOMRIGHT", self.current, "BOTTOMLEFT", -10, 0)
end

local function GetEditable(self)
  return self.editor and self.editor:GetVisible()
end

local function SetEditable(self, editable)
  assert(type(editable) == "boolean", "param 1 must be a boolean!")

  if editable then
    if not self.editor then
      self.editor = UI.CreateFrame("RiftTextfield", self:GetName().."Editor", self)
      self.editor:SetBackgroundColor(0.3, 0.3, 0.3, 1)
      Library.LibSimpleWidgets.SetBorder(self.editor, 1, 0.5, 0.5, 0.5, 1)
      self.editor:SetAllPoints(self.current)
      --[[self.editor.Event.KeyUp = function(editor, key)
        if not key then return end
        if key == "Return" or key == "Tab" then
		  local pos = tonumber(self.editor:GetText())
          self:SetNewPosition(pos)
        end
      end
	  self.editor.Event.KeyFocusLoss = function()
		local pos = tonumber(self.editor:GetText())
		self:SetNewPosition(pos)
      end--]]
	  self.editor:EventAttach(Event.UI.Input.Key.Up, function(event, editor, key)
        if not key then return end
        if key == "Return" or key == "Tab" then
		  local pos = tonumber(self.editor:GetText())
          self:SetNewPosition(pos)
        end
      end, "UMSlider.KeyUp")
	  self.editor:EventAttach(Event.UI.Input.Key.Focus.Loss, function(event, editor, key)
        if not key then return end
        if key == "Return" or key == "Tab" then
		  local pos = tonumber(self.editor:GetText())
          self:SetNewPosition(pos)
        end
      end, "UMSlider.LoseFocus")		  
    end
    self.editor:SetText(tostring(self.slider:GetPosition()*self.step))
    self.current:SetVisible(false)
    self.editor:SetVisible(true)
  else
    if self.editor then
      self.editor:SetVisible(false)
      self.current:SetVisible(true)
    end
  end
end

local function SetNewPosition(self, pos)
	local min, max = self.slider:GetRange()
	max = RoundValue(max*self.step,0)
	if pos ~= nil and pos >= min and pos <= max then
		local newpos = RoundValue(pos/self.step, 0)
		self.slider:SetPosition(newpos)
		self.current:SetText(tostring(RoundValue(pos/self.step,0)))
	else
		self.editor:SetText(tostring(self.slider:GetPosition()*self.step, 0))
	end
	self.editor:SetKeyFocus(false)
end

-- Constructor Function

function umSlider(name, parent)
  local widget = UI.CreateFrame("Frame", name, parent)
  widget.slider = UI.CreateFrame("RiftSlider", widget:GetName().."Slider", widget)
  widget.current = UI.CreateFrame("Text", widget:GetName().."Current", widget)

  widget.enabled = true
  widget.step = 1
  widget.current:SetText(tostring(widget.slider:GetPosition()*widget.step))

  widget.current:SetBackgroundColor(0, 0, 0, 1)
  Library.LibSimpleWidgets.SetBorder(widget.current, 1, 0.5, 0.5, 0.5, 1)

  ResizeToFit(widget)

   widget.slider:EventAttach(Event.UI.Slider.Change, function()
    local pos = tostring(widget.slider:GetPosition()*widget.step)
    widget.current:SetText(tostring(pos))
    if widget.editor then
      widget.editor:SetText(tostring(pos))
    end
  end, widget:GetName().."SliderChange")

  widget.SetBorder = SetBorder
  widget.SetBackgroundColor = SetBackgroundColor
  widget.GetEnabled = GetEnabled
  widget.SetEnabled = SetEnabled
  widget.GetRange = GetRange
  widget.SetRange = SetRange
  widget.GetPosition = GetPosition
  widget.SetPosition = SetPosition
  widget.ResizeToFit = ResizeToFit
  widget.GetEditable = GetEditable
  widget.SetEditable = SetEditable
  widget.SetNewPosition = SetNewPosition

  --Library.LibSimpleWidgets.EventProxy(widget, {"SliderChange","SliderGrab","SliderRelease"})

  return widget
end
